using System;
using System.ComponentModel;
using System.Drawing;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web;

namespace Spaanjaars.Toolkit
{
  /// <summary>
  /// The Rating control displays a number of stars to indicate the popularity of an item.
  /// Additionally, it allows a user to rate an item and fires an event thhat you can handle to register the rate.
  /// </summary>
  [
  DefaultProperty("OneStarImage"),
  DefaultEvent("Rate"),
  ToolboxData("<{0}:ContentRating runat='server' />"),
  ToolboxBitmap(typeof(ContentRating), "Resources.ContentRating.bmp")
  ]
  public class ContentRating : CompositeControl
  {

    #region Private Variables

    private int[] dataSource = new int[] { 0, 0, 0, 0, 0 };
    private string oneStarImage = String.Empty;
    private string halfStarImage = String.Empty;
    private string oneStarImageDisabled = String.Empty;

    #endregion

    #region Events

    //Rating - Fires before the Rate event is raised.
    private static readonly object ratingEventKey = new object();

    /// <summary>
    /// Is fired before the control raises its rating event.
    /// </summary>
    [
      Category("Rating"),
      Description("Is fired before the control raises its rating event.")
    ]
    public event RatingEventHandler Rating
    {
      add { Events.AddHandler(ratingEventKey, value); }
      remove { Events.RemoveHandler(ratingEventKey, value); }
    }

    /// <summary>
    /// Raises the <see cref="E:Rating"/> event.
    /// </summary>
    /// <param name="e">The <see cref="Spaanjaars.Toolkit.RateEventArgs"/> instance containing the event data.</param>
    protected virtual void OnRating(RateEventArgs e)
    {
      RatingEventHandler handler = Events[ratingEventKey] as RatingEventHandler;

      if (handler != null)
        handler(this, e);
    }

    //Rated - After the rate
    private static readonly object ratedEventKey = new object();
    /// <summary>
    /// Is fired when the control is done rating.
    /// </summary>
    [
      Category("Rating"),
      Description("Is fired when the control is done rating.")
    ]
    public event RatingEventHandler Rated
    {
      add { Events.AddHandler(ratedEventKey, value); }
      remove { Events.RemoveHandler(ratedEventKey, value); }
    }

    /// <summary>
    /// Raises the <see cref="E:Rated"/> event.
    /// </summary>
    /// <param name="e">The <see cref="Spaanjaars.Toolkit.RateEventArgs"/> instance containing the event data.</param>
    protected virtual void OnRated(RateEventArgs e)
    {
      RatingEventHandler handler = Events[ratedEventKey] as RatingEventHandler;

      if (handler != null)
        handler(this, e);
    }

    /// <summary>
    /// Determines whether the event for the server control is passed up the page's UI server control hierarchy.
    /// </summary>
    /// <param name="source">The source of the event.</param>
    /// <param name="args">An <see cref="T:System.EventArgs"></see> object that contains the event data.</param>
    /// <returns>
    /// true if the event has been canceled; otherwise, false. The default is false.
    /// </returns>
    protected override bool OnBubbleEvent(object source, EventArgs args)
    {
      bool handled = false;
      CommandEventArgs ce = args as CommandEventArgs;

      if (ce != null && ce.CommandName == "Rate")
      {
        RateEventArgs myArgs = new RateEventArgs(Convert.ToInt32(ce.CommandArgument), true);

        HttpCookie rateCookie;
        string cookieKey = String.Empty;
        string cookieValue = String.Empty;

        // Try to set a cookie to indicate the user has rated this item.
        if (Context.Request.Browser.Cookies == true)
        {
          myArgs.SupportsCookies = true;

          cookieKey = "Rate_" + ItemId.ToString();
          cookieValue = ce.CommandArgument.ToString();

          rateCookie = Context.Request.Cookies["Rate"];

          if (rateCookie == null)
          {
            myArgs.HasRated = false;
          }
          else
          {
            if (rateCookie.Values[cookieKey] == null)
            {
              myArgs.HasRated = false;
            }
            else
            {
              myArgs.HasRated = true;
            }
          }
        }
        else
        {
          myArgs.SupportsCookies = false;
        }

        // Raise the Rating event. 
        OnRating(myArgs);

        if (!myArgs.Cancel)
        {
          // Create a new cookie value for this rate.
          rateCookie = new HttpCookie("Rate");
          rateCookie.Expires = DateTime.Now.AddMonths(6);
          rateCookie.Values.Add(cookieKey, cookieValue);
          Context.Response.Cookies.Add(rateCookie);
          // Raise the Rated event to notify clients.
          OnRated(myArgs);
          handled = true;
        }
      }
      return handled;
    }

    #endregion

    #region Public Properites

    /// <summary>
    /// Gets or sets a unique ID for the item being rated.
    /// </summary>
    /// <value>The item id.</value>
    [
      Bindable(true),
      Category("Data"),
      DefaultValue(""),
      Description("Gets or sets a unique ID for the item being rated.")
    ]
    public object ItemId
    {
      get
      {
        object itemId = ViewState["ItemId"];
        if (itemId != null)
        {
          return (object)itemId;
        }
        else
        {
          return null;
        }
      }
      set
      {
        ViewState["ItemId"] = value;
      }
    }

    /// <summary>
    /// The Data Source for the rating control. An array of ints that specifies the number of rates for each rating value.
    /// An array like this {2, 0, 0, 5, 5} means two users rated the item with two stars, 5 users rated it with four stars and five users rated it with five stars.
    /// </summary>
    [
    Category("Data"),
    Description("The Data Source for the rating control. An array of ints that specifies the number of rates for each rating value."),
    Browsable(false)]
    public int[] DataSource
    {
      get
      {
        return dataSource;
      }
      set
      {
        dataSource = value;
        this.EnsureChildControls();
      }
    }

    /// <summary>
    /// The path to an image that is used to display a single, selected star.
    /// </summary>
    [
    Category("Appearance"),
    Description("The path to an image that is used to display a single, selected star."),
    EditorAttribute(typeof(System.Web.UI.Design.ImageUrlEditor), typeof(System.Drawing.Design.UITypeEditor))
    ]
    public string OneStarImage
    {
      get
      {
        return oneStarImage;
      }
      set
      {
        oneStarImage = value;
        this.EnsureChildControls();
      }
    }

    /// <summary>
    /// The path to an image that is used to display a single, disabled star.
    /// </summary>
    [
    Category("Appearance"),
    Description("The path to an image that is used to display a single, disabled star."),
    EditorAttribute(typeof(System.Web.UI.Design.ImageUrlEditor), typeof(System.Drawing.Design.UITypeEditor))
    ]
    public string OneStarImageDisabled
    {
      get
      {
        return oneStarImageDisabled;
      }
      set
      {
        oneStarImageDisabled = value;
        this.EnsureChildControls();
      }
    }

    /// <summary>
    /// The path to an image that is used to display a star that is partially selected and partially disabled.
    /// </summary>
    [
    Category("Appearance"),
    Description("The path to an image that is used to display a star that is partially selected and partially disabled."),
    EditorAttribute(typeof(System.Web.UI.Design.ImageUrlEditor), typeof(System.Drawing.Design.UITypeEditor))
    ]
    public string HalfStarImage
    {
      get
      {
        return halfStarImage;
      }
      set
      {
        halfStarImage = value;
        this.EnsureChildControls();
      }
    }

    /// <summary>
    /// Gets or sets the text that is displayed below the rating stars. Leave emtpy to hide the legend. You can use {0} and (1} to display the total number of users and the average value respectively.
    /// </summary>
    /// <value>The legend text.</value>
    [
      Bindable(true),
      Category("Appearance"),
      DefaultValue(""),
      Description("Gets or sets the text that is displayed below the rating stars. Leave emtpy to hide the legend. You can use {0} and (1} to display the total number of users and the average value respectively.")
    ]
    public string LegendText
    {
      get
      {
        object legendText = ViewState["LegendText"];
        if (legendText != null)
        {
          return (string)legendText;
        }
        else
        {
          return String.Empty;
        }
      }
      set
      {
        ViewState["LegendText"] = value;
      }
    }

    /// <summary>
    /// Ensures that the control is rendered as an HTML table, not as a span tag.
    /// </summary>
    protected override HtmlTextWriterTag TagKey
    {
      get { return HtmlTextWriterTag.Table; }
    }

    #endregion

    #region Databinding and Control Creation

    /// <summary>
    /// Binds a data source to the <see cref="T:System.Web.UI.WebControls.CompositeControl"></see> and all its child controls.
    /// </summary>
    public override void DataBind()
    {
      if (ItemId == null)
      {
        throw new ArgumentException(@"Can't bind a datasource 
          without a valid ItemId");
      }

      if (dataSource == null)
      {
        dataSource = new int[] { 0, 0, 0, 0, 0 };
      }

      base.DataBind();
      Controls.Clear();
      ClearChildViewState();
      TrackViewState();
      ViewState["RateItems"] = dataSource;
      CreateControlHierarchy(dataSource);
      ChildControlsCreated = true;
    }

    private void CreateControlHierarchy(int[] theDataSource)
    {
      RatingSettings mySetings = new RatingSettings(theDataSource);

      TableRow myTableRow = new TableRow();
      for (int numberOfCells = 1; numberOfCells <= theDataSource.Length; numberOfCells++)
      {
        TableCell myCell = new TableCell();

        ImageButton starImage = new ImageButton();
        starImage.CommandArgument = numberOfCells.ToString();
        starImage.CommandName = "Rate";
        starImage.ID = "Image" + numberOfCells.ToString();

        string imageUrl = String.Empty;
        if (numberOfCells <= mySetings.TruncatedValue)
        {
          imageUrl = GetImageUrl(oneStarImage, Constants.RatingOneStarImage);
        }
        else
        {
          if (numberOfCells == mySetings.TruncatedValue + 1
                    && mySetings.ShowHalfImage)
          {
            imageUrl = GetImageUrl(halfStarImage, Constants.RatingHalfStarImage);
          }
          else
          {
            imageUrl = GetImageUrl(oneStarImageDisabled,
                          Constants.RatingOneStarDisabledImage);
          }
        }
        starImage.ImageUrl = imageUrl;

        myCell.Controls.Add(starImage);
        myTableRow.Cells.Add(myCell);
      }

      this.Controls.Add(myTableRow);

      if (!String.IsNullOrEmpty(LegendText))
      {
        TableRow myLegendTableRow = new TableRow();
        TableCell myLegendeCell = new TableCell();
        myLegendeCell.ColumnSpan = theDataSource.Length;
        myLegendeCell.Text = String.Format(LegendText, mySetings.TotalRates.ToString(), Math.Round(mySetings.Average, 2).ToString());
        myLegendTableRow.Cells.Add(myLegendeCell);
        this.Controls.Add(myLegendTableRow);

      }
      ChildControlsCreated = true;
    }

    /// <summary>
    /// Called by the ASP.NET page framework to notify server controls that use composition-based implementation to create any child controls they contain in preparation for posting back or rendering.
    /// </summary>
    protected override void CreateChildControls()
    {
      Controls.Clear();

      if (ViewState["RateItems"] != null)
      {
        int[] tempDataSource = (int[])ViewState["RateItems"];
        CreateControlHierarchy(tempDataSource);
      }
      else
      {
        if (this.DesignMode)
        {
          // We're displaying on a design surface, so make up a fake datasource.
          int[] tempDataSource = new int[] { 18, 23, 17, 12, 45 };
          CreateControlHierarchy(tempDataSource);
        }
      }
    }
    #endregion

    #region Private Methods

    /// <summary>
    /// Gets an ImageUrl for the images, either from the public ImageUrl properties, or from the images
    /// embedded in the assembly.
    /// </summary>
    /// <param name="imageUrl">The image URL.</param>
    /// <param name="embeddedImage">The path to the embedded image.</param>
    private string GetImageUrl(string imageUrl, string embeddedImage)
    {
      string localImageUrl;
      if (String.IsNullOrEmpty(imageUrl))
      {
        localImageUrl = this.Page.ClientScript.GetWebResourceUrl(this.GetType(), embeddedImage);
      }
      else
      {
        localImageUrl = base.ResolveUrl(imageUrl);
      }
      return localImageUrl;
    }
    #endregion

  }
}
