using System;

namespace Spaanjaars.Toolkit
{
  /// <summary>
  /// The RatingSettings class hides the complexity for calculating the number of rates, average etc.
  /// It exposes simple, read-only properties like <see cref="TruncatedValue"/> and <see cref="Average"/> based on the datasource that 
  /// is passed to its constructor.
  /// </summary>
  internal class RatingSettings
  {

    #region Private Variables

    private double totalScore;
    private double totalRates;
    private double average;
    private double truncatedValue;
    private bool showHalfImage = false;

    #endregion

    #region Constructor(s)

    /// <summary>
    /// Initializes a new instance of the RatingSettings class.
    /// </summary>
    internal RatingSettings(int[] rateItems)
    {
      int loopCount;
      // Calculate Total
      for (loopCount = 0; loopCount < (rateItems.Length); loopCount++)
      {
        totalScore += ((loopCount + 1) * rateItems[loopCount]);
        totalRates += rateItems[loopCount];
      }
      
      if (totalRates == 0)
      {
        average = 0;
      }
      else
      {
        average = totalScore / totalRates;
      }

      truncatedValue = Math.Truncate(average);
      double remainder = (average - truncatedValue) * 10;

      if (remainder > 3 && remainder < 7)
      {
        // Show half image because the value is between x.3 and x.7
        showHalfImage = true;
      }
      else
      {
        if (remainder >= 7)
        {
          // value is greater than x.7 so we round up to the next integer.
          truncatedValue += 1;
        }
      }
    }

    #endregion

    #region Internal Properties

    /// <summary>
    /// Gets a value indicating whether to show a half image or not..
    /// </summary>
    /// <value><c>true</c> if the half image must be displayed, otherwise, <c>false</c>.</value>
    internal bool ShowHalfImage
    {
      get
      {
        return showHalfImage;
      }
    }

    /// <summary>
    /// Gets the truncated value from the rating. This is the whole number of the rate, rounded up to 
    /// the next integer when the initial value is greater than x.7.
    /// </summary>
    /// <value>The truncated value.</value>
    internal double TruncatedValue
    {
      get
      {
        return truncatedValue;
      }
    }

    /// <summary>
    /// Gets the average rating.
    /// </summary>
    /// <value>The average.</value>
    internal double Average
    {
      get
      {
        return average;
      }
    }

    /// <summary>
    /// Gets the total number of rates.
    /// </summary>
    /// <value>The total number of rates.</value>
    internal double TotalRates
    {
      get
      {
        return totalRates;
      }
    }
    #endregion

  }
}
